<?php
require_once("3rdparty/domain/IRegistrar.php");
require_once("3rdparty/domain/standardfunctions.php");
require_once __DIR__ . '/helpers.php';

// Debuggg
ini_set('display_errors',true);
error_reporting(E_ALL);

class xynta implements IRegistrar
{
    public $User;
    public $Password;
    public $Testmode;

    public $Error;
    public $Warning;
    public $Success;

    public $Period = 1;
    public $registrarHandles = array();

    private $ClassName;

    /**
     * @var object Api Xynta
     */
    private $Api_Xynta = "";

    /**
     * xYnta constructor.
     */
    public function __construct()
    {
        $this->ClassName = __CLASS__;

        $this->Error = array();
        $this->Warning = array();
        $this->Success = array();
    }


    /**
     * Call the Api, excute a action with parameters and return result
     *
     * @param string $module with module name
     * @param string $action with action name
     * @param array $params with actions
     * @param string $returntype type of return (XML, JSON etc)
     */
    private function callApi($module, $action, $params = array(), $returntype = "XML") {
        // If not initialised, then create object
        // We don't do this in the constructor because we don't know when the user and pw are set
        if($this->Api_Xynta == "") {
            $this->Api_Xynta = new Api_Xynta();
            $this->Api_Xynta->setLogin($this->User, $this->Password, $this->Testmode);
        }
        // Call api and return response
        try {
            $response = $this->Api_Xynta->execute($module, $action, $params, $returntype);
            return $response;
        } catch(Api_Odr_Exception $e) {
            $this->Error[] = $e->getMessage();
            return false;
        }

    }


    /**
     * Get class version information
     *
     * @return arrays
     * @static
     */
    public static function getVersionInformation() {
        $version = array();
        require_once("3rdparty/domain/xynta/version.php");
        return $version;
    }

    /**
     * Check whether a domain is already registered or not
     *
     * @param string $domain The name of the domain that needs to be checked
     * @return bool True if free, False if not free, False and $this->Error[] in case of error
     */
    public function checkDomain($domain) {
        // Split domain
        $aDomain = $this->extractSldAndTldFromString($domain);

        // Call Api
        $params = array(
            'SLD' => $aDomain['SLD'],
            'TLD' => $aDomain['TLD']
        );
        $oXML = $this->callApi('domains','check', $params);

        // If response is false, return it
        if($oXML === false) {
            $this->Error[] = 'xYnta: No valid API response.';
            return false;
        }

        // Check if we got a valid response
        if(!isset($oXML->Domains_Check->Status)) {
            $this->Error[] = 'xYnta: API error while checking TLD.';
            return false;
        }
        // Success1 = Available
        else if ($oXML->Domains_Check->Status == 'Success1') {
            return true;
        }
        // Success2 = Not available
        elseif ($oXML->Domains_Check->Status == 'Success2') {
            return false;
        }
        // Not available
        else {
            $this->Error[] = 'xYnta: Checking availability for domain TLD is not supported via the API.';
            return false;
        }
    }

    /**
     * Register a new domain
     *
     * @param string $domain      The domainname that needs to be registered
     * @param array  $nameservers The nameservers for the new domain
     * @param Whois  $whois       The customer information for the domain's whois information
     * @param string $authcode    Authorisation code for domain
     * @return bool
     */
    public function registerDomain($domain, $nameservers = array(), $whois = null, $authcode = '') {
        $aDomain = $this->extractSldAndTldFromString($domain);

        $ownerHandle = $this->_obtainHandle($domain, $whois, HANDLE_OWNER);
        if (!$ownerHandle) {
            return $ownerHandle;
        }

        $adminHandle = $this->_obtainHandle($domain, $whois, HANDLE_ADMIN);
        if (!$adminHandle) {
            return $adminHandle;
        }

        $techHandle = $this->_obtainHandle($domain, $whois, HANDLE_TECH);
        if (!$techHandle) {
            return $techHandle;
        }

        $getDNS1 = $nameservers['ns1'];
        $getDNS2 = $nameservers['ns2'];
        $getDNS3 = $nameservers['ns3'];

        // First step, always empty cart
        $this->callApi('domains','empty_cart');

        // Second step, add to cart
        $params = array(
            'SLD' => $aDomain['SLD'],
            'TLD' => $aDomain['TLD'],
            'OwnerHandle' => $ownerHandle,
            'AdminHandle' => $adminHandle,
            'TechHandle' => $techHandle,
            'DNS1' => $getDNS1,
            'DNS2' => $getDNS2,
            'DNS3' => $getDNS3
        );
        // Got an Authcode?
        if($authcode != "") {
            $params['EPP'] = $authcode;
        }
        $oXMLaddto = $this->callApi('domains','add_to_cart', $params);

        if ($oXMLaddto->Domains_Add_To_Cart->Status == "Success1") {

            // Third step, order it
            $oXMLorderCart = $this->callApi('domains', 'order_cart');

            if ($oXMLorderCart->Domains_Order_Cart->Status == "Success1") {
                return true;
            }
            else {
                $getError = $oXMLaddto->Domains_Order_Cart->Description;
                $getErrorb = $oXMLaddto->Domains_Order_Cart;

                $this->Error[] = sprintf("xYnta: Cant order cart '%s': ", "$getError ($getErrorb)");
                return false;
            }
        }
        else {
            $getError = $oXMLaddto->Domains_Add_To_Cart->Description;
            $getErrorb = $oXMLaddto->Domains_Add_To_Cart;

            $this->Error[] = sprintf("xYnta: Cant add domain to cart '%s': ", "$getError ($getErrorb)");
            return false;
        }
    }

    /**
     * Transfer a domain to the given user
     *
     * @param string $domain      Domain name for transfer
     * @param array  $nameservers The nameservers for the transfered domain
     * @param Whois  $whois       The contact information for the new owner, admin, tech and billing contact
     * @param string $authcode    Authorisation code for domain
     * @return bool
     */
    public function transferDomain($domain, $nameservers = array(), $whois = null, $authcode = '') {
        return $this->registerDomain($domain, $nameservers, $whois, $authcode);
    }

    /**
     * Get a list of all the domains
     *
     * @param string $contactHandle The handle of a contact, so the list could be filtered (useful for updating domain whois data)
     * @return array|bool
     * @todo: implement $contactHandle filter
     */
    public function getDomainList($contactHandle = "") {
        // Contacts
        $contactlisttmp = $this->getContactList();
        $contactlist = array();
        foreach($contactlisttmp as $contact) {
            $contactlist[$contact['Handle']] = $contact;
        }

        // Get domain list
        $oDomains = $this->callApi('domains','list');
        if ($oDomains->Domains_List->Status == "Success1") {
            $domain_array = array();

            foreach ($oDomains->Domains_List->Domains->Item as $oDomain) {
//                echo "<pre>".print_r($oDomain, true)."</pre>";

                // Call Api
                $params = array(
                    'SLD' => (string) $oDomain->SLD,
                    'TLD' => (string) $oDomain->TLD
                );
                $oDomainDetails = $this->callApi('domains','get_details', $params);
//                echo "<pre>".print_r($oDomainDetails, true)."</pre>";

                if($oDomainDetails->Domains_Get_Details->Status == "Success1") {
                    $oDomainDetails = $oDomainDetails->Domains_Get_Details;

                    $getDNS1 = (string) $oDomainDetails->DNS->DNS1;
                    $getDNS2 = (string) $oDomainDetails->DNS->DNS2;
                    $getDNS3 = (string) $oDomainDetails->DNS->DNS3;
                    $nameservers_array = array($getDNS1, $getDNS2, $getDNS3);

                    $whois = new whois();
                    $whois->ownerHandle = (string) $oDomainDetails->Contacts->OwnerHandle;
                    $whois->adminHandle = (string) $oDomainDetails->Contacts->AdminHandle;
                    $whois->techHandle = (string) $oDomainDetails->Contacts->TechHandle;

                    $aTypes = array("owner", "admin", "tech");
                    foreach($aTypes as $sType) {
                        if(isset($contactlist[$whois->{$sType."Handle"}])) {
                            $Handle = $contactlist[$whois->{$sType."Handle"}];
                            foreach($Handle as $field => $value) {
                                $whois->{$sType.$field} = $value;
                            }
                        }
                    }

                    $expirationdate = date("Y-m-d",strtotime((string) $oDomainDetails->General->ExpirationDate));
                    $registrationdate = date("Y-m-d",strtotime((string) $oDomainDetails->General->RegistrationDate));
                    $authkey = "";

                    $domaindetails = array(
                        "Domain" => $oDomain->SLD.".".$oDomain->TLD,
                        "Information" => array(
                            "nameservers" => $nameservers_array,
                            "whois" => $whois,
                            "expiration_date" => $expirationdate,
                            "registration_date" => $registrationdate,
                            "authkey" => $authkey
                        )
                    );
                    $domain_array[] = $domaindetails;

                }
            }
            return $domain_array;
        }
        else {
            $this->Error[] = sprintf("xYnta: Error while retreiving domains: %s", print_r($oXML, true));
            return false;
        }
    }

    /**
     * Get all available information of the given domain
     *
     * @param mixed $domain The domain for which the information is requested
     * @return array|bool
     */
    public function getDomainInformation($domain) {
        // Split domain
        $aDomain = $this->extractSldAndTldFromString($domain);

        // Call Api
        $params = array(
            'SLD' => $aDomain['SLD'],
            'TLD' => $aDomain['TLD']
        );
        $oDomainDetails = $this->callApi('domains','get_details', $params);
//                echo "<pre>".print_r($oDomainDetails, true)."</pre>";

        if($oDomainDetails->Domains_Get_Details->Status == "Success1") {
            $oDomainDetails = $oDomainDetails->Domains_Get_Details;

            $getDNS1 = (string) $oDomainDetails->DNS->DNS1;
            $getDNS2 = (string) $oDomainDetails->DNS->DNS2;
            $getDNS3 = (string) $oDomainDetails->DNS->DNS3;
            $nameservers_array = array($getDNS1, $getDNS2, $getDNS3);

            $whois = new whois();
            $whois->ownerHandle = (string) $oDomainDetails->Contacts->OwnerHandle;
            $whois->adminHandle = (string) $oDomainDetails->Contacts->AdminHandle;
            $whois->techHandle = (string) $oDomainDetails->Contacts->TechHandle;

            $expirationdate = date("Y-m-d",strtotime((string) $oDomainDetails->General->ExpirationDate));
            $registrationdate = date("Y-m-d",strtotime((string) $oDomainDetails->General->RegistrationDate));
            $authkey = "";

            $domaindetails = array(
                "Domain" => $domain,
                "Information" => array(
                    "nameservers" => $nameservers_array,
                    "whois" => $whois,
                    "expiration_date" => $expirationdate,
                    "registration_date" => $registrationdate,
                    "authkey" => $authkey
                )
            );
            return $domaindetails;
        }
        else {
            $error_msg = $oDomainDetails->Domains_Get_Details->Description;
            $this->Error[] = sprintf("xYnta: Error while retreiving information about '%s': %s", $domain, $error_msg);
            return false;
        }
    }

    /**
     * Get EPP code/token
     *
     * @param mixed $domain Domain name
     * @return null|string|bool
     */
    public function getToken($domain) {
        // Split domain
        $aDomain = $this->extractSldAndTldFromString($domain);

        // Call Api
        $params = array(
            'SLD' => $aDomain['SLD'],
            'TLD' => $aDomain['TLD']
        );
        $oXML = $this->callApi('domains','request_epp', $params);
        if ($oXML->Domains_Request_EPP->Action1->Status == "Success1") {
            return true;
        }
        else {
            $error_msg = $oXML->Domains_Request_EPP->Action1->Description;
            $this->Error[] = sprintf("xYnta: Error while retreiving authorization code '%s': %s", $domain, $error_msg);
            return false;
        }
    }

    /**
     * Check domain information for one or more domains
     *
     * @param array $domains Array with list of domains. Key is domain, value must be filled
     * @return array
     */
    public function getSyncData($domains) {
        $limit   = 10;
        $checked = 0;

        foreach ($domains as $domain => &$value) {
            $data = $this->getDomainInformation($domain);

            if ($data === false) {
                $value['Status']    = 'error';
                $value['Error_msg'] = 'Either domain not found or internal error happened';

                continue;
            }

            $checked++;

            if ($limit !== null && $checked > $limit) {
                break;
            }

            /** @var array $data  Because PHPStorm */
            /** @var array $value Because PHPStorm */

            $value = $data;

            $value['Information']['nameservers']     = $data['Information']['nameservers'];
            $value['Information']['expiration_date'] = isset($data['Information']['expiration_date']) ? $data['Information']['expiration_date'] : '';
            $value['Information']['auto_renew']      = empty($data['Information']['auto_renew']) ? '' : $data['Information']['auto_renew'];

            $value['Status'] = 'success';
        }

        unset($value);

        // Return list  (domains which aren't completed with data, will be synced by a next cronjob)
        return $domains;
    }

    /**
     * Delete a domain
     *
     * @param string $domain  The name of the domain that you want to delete
     * @param string $delType end|now
     * @return bool Was domain successfully removed or not
     */
    public function deleteDomain($domain, $delType = 'end') {
        // Split domain
        $aDomain = $this->extractSldAndTldFromString($domain);

        // Call Api
        $params = array(
            'SLD' => $aDomain['SLD'],
            'TLD' => $aDomain['TLD']
        );
        if($delType == "end") {
            $oXML = $this->callApi('domains', 'cancel', $params);
        }
        else {
            $oXML = $this->callApi('domains', 'cancel_direct', $params);
        }

        if ($oXML->Domains_Cancel->Action1->Status == "Success1") {
            return true;
        }
        else {
            $error_msg = $oXML->Domains_Cancel->Action1->Description;
            $this->Error[] = sprintf("xYnta: Error while terminating '%s': %s", $domain, $error_msg);
            return false;
        }
    }

    /**
     * Change the autorenew state of the given domain. When autorenew is enabled, the domain will be extended
     *
     * @param string $domain    The domain name to change the autorenew setting for
     * @param bool   $autorenew The new autorenew value (True = On|False = Off)
     * @return bool True when the setting is succesfully changed, False otherwise
     * @todo: when the api call has been created, use it.
     */
    public function setDomainAutoRenew($domain, $autorenew = true) {
        // Not available yet
        $this->Error[] = sprintf("xYnta: This function is not available.");
        return false;

        // Split domain
        $aDomain = $this->extractSldAndTldFromString($domain);

        // Call Api
        $params = array(
            'SLD' => $aDomain['SLD'],
            'TLD' => $aDomain['TLD']
        );
        $oXML = $this->callApi('domains','cancel', $params);

        if ($oXML->Domains_Cancel->Action1->Status == "Success1") {
            return true;
        }
        else {
            $this->Error[] = sprintf("xYnta: Domainnames are automatically renewed unless canceled");
            return false;
        }
    }

    /**
     * Change the lock status of the specified domain
     * @param string $domain The domain to change the lock state for
     * @param bool   $isLock New lock state
     * @return bool True is the lock state was changed succesfully
     * @todo: Isn't realy working right now, we can't set the lock state, but only toggle it.
     */
    public function lockDomain($domain, $lock = true) {
        // Split domain
        $aDomain = $this->extractSldAndTldFromString($domain);

        // Call Api
        $params = array(
            'SLD' => $aDomain['SLD'],
            'TLD' => $aDomain['TLD']
        );
        $oXML = $this->callApi('domains', 'edit_lock', $params);

        if ($oXML->Domains_Edit_Lock->Action1->Status == "Success1") {
            return true;
        }
        else {
            $error_msg = $oXML->Domains_Edit_Lock->Action1->Description;
            $this->Error[] = sprintf("xYnta: Error while (un)locking '%s': %s", $domain, $error_msg);
            return false;
        }
    }



    /**
     * Create a new whois contact
     *
     * @param Whois $whois The whois information for the new contact
     * @param string $type  The contact type. This is only used to access the right data in the $whois object
     * @return bool Is new contact was created succesfully or not
     * @throws Api_Odr_Api_Xynta_ExceptionException
     */
    public function createContact($whois, $type = HANDLE_OWNER) {
        // Generate contact
        $contact = $this->_mapWhoisToContact($whois, $type);

        // Call Api
        $oXML = $this->callApi('domains','add_contact', $contact);

        // If response is false, return it
        if($oXML === false) {
            $this->Error[] = 'xYnta: No valid API response.';
            return false;
        }

        if ($oXML->Domains_Add_Contact->Status == "Success1") {
            $handle = (string) $oXML->Domains_Add_Contact->Output->HandleIdentifier;
            return $handle;
        }
        else {
            $error_msg = $oXML->Domains_Add_Contact->Description;
            $this->Error[] = sprintf("xYnta: Error while creating contact: %s", $type);
            $this->Error[] = sprintf("xYnta: Error while creating contact: %s", $error_msg);
            $this->Error[] = sprintf("xYnta: Error while creating contact: %s", print_r($contact, true));
            $this->Error[] = sprintf("xYnta: Error while creating contact: %s", print_r($oXML, true));
            $this->Error[] = sprintf("xYnta: Requested url: %s", $this->Api_Xynta->last_request_url);
            return false;
        }
    }

    /**
     * Update the whois information for the given contact person
     * @param string $handle The handle of the contact to be changed
     * @param Whois  $whois  The new whois information for the given contact
     * @param mixed  $type   The of contact. This is used to access the right fields in the whois array
     * @return bool
     */
    public function updateContact($handle, $whois, $type = HANDLE_OWNER) {
       // Generate contact
        $contact = $this->_mapWhoisToContact($whois, $type);
        $contact["HandleIdentifier"] = $handle;

        // Call Api
        $oXML = $this->callApi('domains','edit_contact', $contact);

        if ($oXML->Domains_Edit_Contact->Status == "Success1") {
            return true;
        }
        else {
            $error_msg = $oXML->Domains_Edit_Contact->Description;
            $this->Error[] = sprintf("xYnta: Error while editting contact: %s", $error_msg);
            $this->Error[] = sprintf("xYnta: contact: %s", print_r($contact,true));
            $this->Error[] = sprintf("xYnta: Whois: %s", print_r($whois,true));
            return false;
        }
    }

    /**
     * Get a list of contact handles available
     *
     * @param string $surname Surname to limit the number of records in the list
     * @return array|bool List of all contact matching the $surname search criteria
     * @throws Api_Xynta_Exception
     */
    public function getContactList($surname = "")
    {
        // Default, no results
        $contact_list = array();

        // Call Api
        $oXML = $this->callApi('domains','list_contact');
//        echo "<pre>".print_r($oXML, true)."</pre><br />";
//        exit;

        // If response is false, return it
        if($oXML === false) {
            $this->Error[] = 'xYnta: No valid API response.';
            return false;
        }

        // Do we got any results?
        if(!isset($oXML->Domains_List_Contact->Results->Item)) {
            $this->Error[] = 'xYnta: Error in calling the API.';
            return false;
        }
        else {
            foreach ($oXML->Domains_List_Contact->Results->Item as $oResultContact) {
                if ($surname != "") {
                    if (strpos($oResultContact->SurNam, $surname) !== false) {
                        $contact = array();
                        $contact['Handle'] = (string) $oResultContact->HandleIdentifier;
                        if ((string) $oResultContact->CompanyName != "") {
                            $contact['CompanyName'] =(string) $oResultContact->CompanyName;
                        }
                        $contact['SurName'] = (string) $oResultContact->SurName;
                        $contact['Initials'] = (string) $oResultContact->Initials;
                        $contact_list[] = $contact;
                    }
                }
                else {
                    $contact = array();
                    $contact['Handle'] = (string) $oResultContact->HandleIdentifier;
                    if ((string) $oResultContact->CompanyName != "") {
                        $contact['CompanyName'] =(string) $oResultContact->CompanyName;
                    }
                    $contact['SurName'] = (string) $oResultContact->SurName;
                    $contact['Initials'] = (string) $oResultContact->Initials;
                    $contact_list[] = $contact;
                }
            }
        }
//        echo "<pre>".print_r($contact_list, true)."</pre><br />";
        if (count($contact_list) > 0) {
            return $contact_list;
        } else {
            return false;
        }
    }

    /**
     * Get information available of the requested contact
     *
     * @param string $handle The handle of the contact to request
     * @return Whois|bool Information available about the requested contact
     */
    public function getContact($handle) {
        $whois = new whois();

        // Call Api
        $params = array(
            'HandleIdentifier' => $handle
        );
        $oXML = $this->callApi('domains','get_contact', $params);
//        echo "<pre>".print_r($oXML, true)."</pre><br />";

        // If response is false, return it
        if($oXML === false) {
            $this->Error[] = 'xYnta: No valid API response.';
            return false;
        }

        // Do we got any results?
        if(!isset($oXML->Domains_Get_Contact->Status)) {
            $this->Error[] = 'xYnta: Error in calling the API.';
            return $whois;
        }
        else {
            if($oXML->Domains_Get_Contact->Status == "Success1") {

                $oContact = $oXML->Domains_Get_Contact->Output;

                $whois->ownerCompanyName      = (string) $oContact->CompanyName;
                $whois->ownerCompanyNumber    = empty($oContact->CompanyNumber) ? null : (string) $oContact->CompanyNumber;
                $whois->ownerTaxNumber        = empty($oContact->TaxNumber) ? null : (string) $oContact->TaxNumber;
                $whois->ownerCompanyLegalForm = (string) $oContact->LegalForm;

                $whois->ownerSex      = (string) $oContact->Gender;
                $whois->ownerInitials = (string) $oContact->FirstName;
                $whois->ownerSurName  = (string) $oContact->LastName;
                $whois->ownerAddress  = (string) $oContact->Address;
                $whois->ownerZipCode  = (string) $oContact->ZipCode;
                $whois->ownerCity     = (string) $oContact->City;

                $whois->ownerCountry      = (string) $oContact->Country;
                $whois->ownerEmailAddress = (string) $oContact->EmailAddress;
                $whois->ownerPhoneNumber  = (string) $oContact->PhoneNumber;
                $whois->ownerFaxNumber    = (string) $oContact->FaxNumber;

                return $whois;

            }
            else {
                $this->Error[] = sprintf("xYnta: Error while retreiving contact: %s", $oXML->Domains_Get_Contact->Description);
                return $whois;
            }

        }
        /** Old
        if ($response) {
            $whois->ownerCompanyName = htmlspecialchars($array['Domains_Get_Contact']['Output']['CompanyName']);
            $whois->ownerInitials = htmlspecialchars($array['Domains_Get_Contact']['Output']['FirstName']);
            $whois->ownerSurName = htmlspecialchars($array['Domains_Get_Contact']['Output']['LastName']);
            $whois->ownerAddress = htmlspecialchars($array['Domains_Get_Contact']['Output']['Address']);
            $whois->ownerZipCode = htmlspecialchars($array['Domains_Get_Contact']['Output']['ZipCode']);
            $whois->ownerCity = htmlspecialchars($array['Domains_Get_Contact']['Output']['City']);

            $whois->ownerCountry = htmlspecialchars($array['Domains_Get_Contact']['Output']['Country']);
            $whois->ownerPhoneNumber = htmlspecialchars($array['Domains_Get_Contact']['Output']['PhoneNumber']);
            $whois->ownerFaxNumber = htmlspecialchars($array['Domains_Get_Contact']['Output']['FaxNumber']);
            $whois->ownerEmailAddress = htmlspecialchars($array['Domains_Get_Contact']['Output']['EmailAddress']);

            $whois->adminCompanyName = htmlspecialchars($array['Domains_Get_Contact']['Output']['CompanyName']);
            $whois->adminInitials = htmlspecialchars($array['Domains_Get_Contact']['Output']['FirstName']);
            $whois->adminSurName = htmlspecialchars($array['Domains_Get_Contact']['Output']['LastName']);
            $whois->adminAddress = htmlspecialchars($array['Domains_Get_Contact']['Output']['Address']);
            $whois->adminZipCode = htmlspecialchars($array['Domains_Get_Contact']['Output']['ZipCode']);
            $whois->adminCity = htmlspecialchars($array['Domains_Get_Contact']['Output']['City']);

            $whois->adminCountry = htmlspecialchars($array['Domains_Get_Contact']['Output']['Country']);
            $whois->adminPhoneNumber = htmlspecialchars($array['Domains_Get_Contact']['Output']['PhoneNumber']);
            $whois->adminFaxNumber = htmlspecialchars($array['Domains_Get_Contact']['Output']['FaxNumber']);
            $whois->adminEmailAddress = htmlspecialchars($array['Domains_Get_Contact']['Output']['EmailAddress']);

            $whois->techCompanyName = htmlspecialchars($array['Domains_Get_Contact']['Output']['CompanyName']);
            $whois->techInitials = htmlspecialchars($array['Domains_Get_Contact']['Output']['FirstName']);
            $whois->techSurName = htmlspecialchars($array['Domains_Get_Contact']['Output']['LastName']);
            $whois->techAddress = htmlspecialchars($array['Domains_Get_Contact']['Output']['Address']);
            $whois->techZipCode = htmlspecialchars($array['Domains_Get_Contact']['Output']['ZipCode']);
            $whois->techCity = htmlspecialchars($array['Domains_Get_Contact']['Output']['City']);

            $whois->techCountry = htmlspecialchars($array['Domains_Get_Contact']['Output']['Country']);
            $whois->techPhoneNumber = htmlspecialchars($array['Domains_Get_Contact']['Output']['PhoneNumber']);
            $whois->techFaxNumber = htmlspecialchars($array['Domains_Get_Contact']['Output']['FaxNumber']);
            $whois->techEmailAddress = htmlspecialchars($array['Domains_Get_Contact']['Output']['EmailAddress']);

            return $whois;
        } else {
            $this->Error[] = sprintf("xYnta: Error while retreiving contact: %s", $error_msg);
            return $whois;
        }
         *  */
    }

    /**
     * Get the handle of a contact
     *
     * @param Whois  $whois The whois information of contact
     * @param string $type  The type of person. This is used to access the right fields in the whois object
     * @return string handle of the requested contact; False if the contact could not be found
     * @throws Api_Odr_Exception
     */
    public function getContactHandle($whois = array(), $type = HANDLE_OWNER)
    {
        $prefix = $this->_getContactPrefix($type);
        $contacts = $this->getContactList($whois->{$prefix . 'SurName'});
        $toCheck = array();
        if (empty($contacts)) {
            return false;
        }
        foreach ($contacts as $contact) {
            if (strpos($contact['CompanyName'], trim($whois->{$prefix . 'Initials'} . ' ' . $whois->{$prefix . 'SurName'})) !== false) {
                $toCheck[] = $contact['Handle'];
            }
        }
        $contacts = $toCheck;
        foreach ($contacts as $contactId) {
            $contact = $this->getContact($contactId);
            if (
                $whois->{$prefix . 'SurName'} === $contact->{$prefix . 'SurName'} &&
                $whois->{$prefix . 'CompanyName'} === $contact->{$prefix . 'CompanyName'} &&
                $whois->{$prefix . 'EmailAddress'} === $contact->{$prefix . 'EmailAddress'}
            ) {
                return $contactId;
            }
        }
        return false;
    }

    /**
     * Returns domain whois handles
     *
     * @param mixed $domain Domain name
     * @return array|bool Domain info with handles
     */
    public function getDomainWhois($domain) {
        // Split domain
        $aDomain = $this->extractSldAndTldFromString($domain);

        // Call Api
        $params = array(
            'SLD' => $aDomain['SLD'],
            'TLD' => $aDomain['TLD']
        );
        $oXML = $this->callApi('domains','get_details', $params);

        $contacts = array();
        $contacts['ownerHandle'] = (string) $oXML->Domains_Get_Details->Contacts->OwnerHandle;
        $contacts['adminHandle'] = (string) $oXML->Domains_Get_Details->Contacts->AdminHandle;
        $contacts['techHandle'] = (string) $oXML->Domains_Get_Details->Contacts->TechHandle;

        return $contacts;
    }

    /**
     * Update the domain Whois data, but only if no handles are used by the registrar
     *
     * @param mixed $domain
     * @param Whois $whois
     * @return bool
     */
    public function updateDomainWhois($domain, $whois) {
        $ownerHandle = $this->_obtainHandle($domain, $whois, HANDLE_OWNER);
        if (!$ownerHandle) {
            return false;
        }

        $adminHandle = $this->_obtainHandle($domain, $whois, HANDLE_ADMIN);
        if (!$adminHandle) {
            return false;
        }

        $techHandle = $this->_obtainHandle($domain, $whois, HANDLE_TECH);
        if (!$techHandle) {
            return false;
        }

        // Split domain
        $aDomain = $this->extractSldAndTldFromString($domain);

        // Call Api
        $params = array(
            'SLD' => $aDomain['SLD'],
            'TLD' => $aDomain['TLD'],
            'OwnerHandle' => $ownerHandle,
            'AdminHandle' => $adminHandle,
            'TechHandle' => $techHandle
        );
        $oXML = $this->callApi('domains','edit_details', $params);

        if ($oXML->Domains_Edit_Contacts->Action1->Status == "Success1") {
            return true;
        }
        else {
            $error_msg = $oXML->Domains_Edit_Contacts->Action1->Description;
            $this->Error[] = sprintf("xYnta: Error while changing contact for '%s': %s", $domain, $error_msg);
            return false;
        }
    }



    /**
     * Update the nameservers for the given domain
     *
     * @param string $domain      The domain to be changed
     * @param array  $nameservers The new set of nameservers
     * @return bool Was update successful or not
     */
    public function updateNameServers($domain, $nameservers = array()) {
        // Split domain
        $aDomain = $this->extractSldAndTldFromString($domain);

        // Nameservers
        $getDNS1 = $nameservers["ns1"];
        $getDNS2 = $nameservers["ns2"];
        $getDNS3 = $nameservers["ns3"];

        // Call Api
        $params = array(
            'SLD'   => $aDomain['SLD'],
            'TLD'   => $aDomain['TLD'],
            'DNS1'  => $getDNS1,
            'DNS2'  => $getDNS2,
            'DNS3'  => $getDNS3
        );
        $oXML = $this->callApi('domains', 'edit_dns', $params);


        if ($oXML->Domains_Edit_DNS->Action1->Status == "Success1") {
            return true;
        }
        else {
            $error_msg = $oXML->Domains_Edit_DNS->Action1->Description;
            $this->Error[] = sprintf("xYnta: Error while changing nameservers for '%s': %s", $domain, $error_msg);
            return false;
        }
    }




    /**
     * Maps WeFact contact types to Xynta contact types
     *
     * @param string $type Contact type
     * @return string $prefix prefix
     * @protected
     */
    protected function _getContactPrefix($type)
    {
        $prefix = '';
        // Determine which contact type should be found
        switch ($type) {
            case HANDLE_OWNER:
                $prefix = 'owner';
                break;
            case HANDLE_ADMIN:
                $prefix = 'admin';
                break;
            case HANDLE_TECH:
                $prefix = 'tech';
                break;
            default:
                break;
        }
        return $prefix;
    }

    /**
     * Maps WeFact Whois object to Xynta contact format
     *
     * @param Whois $whois Whois object
     * @param mixed $type  Contact Type
     * @return array
     */
    protected function _mapWhoisToContact(Whois $whois, $type) {
        $prefix = $this->_getContactPrefix($type);

        $contact = array();
        $contact['Handle'] = $whois->{$prefix.'Handle'};
        if($contact['Handle'] == "" && isset($whois->{$prefix.'WeFactHandle'})) {
            $contact['Handle'] = $whois->{$prefix.'WeFactHandle'};
        }
        $contact['CompanyName'] = $whois->{$prefix.'CompanyName'};
        if($contact['CompanyName'] != "") {
            $contact['CompanyNumber'] = $whois->{$prefix . 'CompanyNumber'};
            $contact['LegalForm'] = $whois->{$prefix . 'CompanyLegalForm'};
            $contact['TaxNumber'] = $whois->{$prefix . 'TaxNumber'};
        }
        else {
            $contact['CompanyNumber'] = "";
            $contact['LegalForm'] = "";
            $contact['TaxNumber'] = "";
        }
        $contact['Gender'] = $whois->{$prefix.'Sex'};
        $contact['FirstName'] = $whois->{$prefix.'Initials'};
        $contact['LastName'] = $whois->{$prefix.'SurName'};
        $contact['Address'] = $whois->{$prefix.'Address'};
        $contact['ZipCode'] = $whois->{$prefix.'ZipCode'};
        $contact['City'] = $whois->{$prefix.'City'};
        $contact['Country'] = $whois->{$prefix.'Country'};
        $contact['PhoneNumber'] = $whois->{$prefix.'PhoneNumber'};
        $contact['FaxNumber'] = $whois->{$prefix.'FaxNumber'};
        $contact['EmailAddress'] = $whois->{$prefix.'EmailAddress'};
        return $contact;
    }

    /**
     * Obtains single handle for domain
     *
     * @param string $domain Domain name
     * @param Whois  $whois  Instance of "whois" class
     * @param string $key    Defined variable (HANDLE_OWNER, HANDLE_ADMIN, HANDLE_TECH, etc)
     *
     * @return bool|string
     *
     * @throws Api_Odr_Exception
     *
     * @protected
     */
    protected function _obtainHandle($domain, $whois, $key, $isTest = false) {
        $prefix = $this->_getContactPrefix($key);

        foreach (self::getRequiredContactFields() as $field => $translation) {
            if (empty($whois->{$prefix . $field})) {
                $this->Error[] = 'Please fill contact\'s ' . $translation . ' (' . $field . ')';
            }
        }

        if (!empty($this->Error)) {
            return false;
        }

//        $this->Error[] = "xYnta: ".print_r($whois, true);

        if ($isTest) {

        }
        // Check if a registrar-specific handle for this domain already exists
        if ($whois !== null && ((isset($whois->{$prefix . 'RegistrarHandles'}[$this->ClassName]) && is_numeric($whois->{$prefix . 'RegistrarHandles'}[$this->ClassName])) || (!empty($this->registrarHandlesMapped[$whois->{$prefix . 'WeFactHandle'}]) && is_numeric($this->registrarHandlesMapped[$whois->{$prefix . 'WeFactHandle'}])))) {
//            $this->Error[] = "xYnta: Check if a registrar-specific handle for this domain already exists => Yes.";
            $handle = empty($whois->{$prefix . 'RegistrarHandles'}[$this->ClassName]) ? $this->registrarHandlesMapped[$whois->{$prefix . 'WeFactHandle'}] : $whois->{$prefix . 'RegistrarHandles'}[$this->ClassName];

            if (empty($whois->{$prefix . 'RegistrarHandles'}[$this->ClassName])) {
                $whois->{$prefix . 'RegistrarHandles'}[$this->ClassName] = $this->registrarHandlesMapped[$whois->{$prefix . 'WeFactHandle'}];
            }
        }
        // If not, check if WHOIS-data for contact is available to search or create new handle
        elseif ($whois !== null && $whois->{$prefix . 'SurName'} != '') {
//            $this->Error[] = "xYnta: If not, check if WHOIS-data for contact is available to search or create new handle => Jup";

            // Search for existing handle, based on WHOIS data
            $handle = $this->getContactHandle($whois, $key);

            // If no existing handle is found, create new handle
            if (!$handle) {
                $handle = $this->createContact($whois, $key);
            }
            if (!$handle) {
                $this->Error[] = "xYnta: prefix: {$prefix}";
                $this->Error[] = "xYnta: Whois".print_r($whois, true);
                $this->Error[] = "xYnta: Handle:".print_r($handle, true);
                $this->Error[] = "xYnta: registrarHandles:".print_r($this->registrarHandles, true);
                $this->Error[] = "xYnta: registrarHandlesMapped:".print_r($this->registrarHandlesMapped, true);
                return false;
            }

            $whois->{$prefix . 'RegistrarHandles'}[$this->ClassName] = $handle;

            // If a new handle is created or found, store in array. WeFact will store this data, which will result in faster registration next time
            $this->registrarHandles[$prefix]                                  = $handle;
            $this->registrarHandlesMapped[$whois->{$prefix . 'WeFactHandle'}] = $handle;
        } else {
            // If no handle can be created, because data is missing, quit function
            return $this->parseError(sprintf("No domain {$prefix} contact given for domain '%s'.", $domain));
        }

        return $handle;
    }

    /**
     * List of required contact fields
     * Format is - key = name in WeFact, value - translation name
     * Warning! Names must be without prefixes
     *
     * @return array
     */
    static public function getRequiredContactFields()
    {
        return array(
            'PhoneNumber'  => 'Phone number',
            'EmailAddress' => 'Email address',
            'ZipCode'      => 'Postal code',
            'Country'      => 'Country',
            'City'         => 'City',
            'Initials'     => 'First name/initials',
            //'SurName'      => 'Surname',
        );
    }

    /**
     * Exact the Sld and Tld from a domain name string
     *
     * @param string $domain domain name
     * @return array $aReturn array with SLD en TLD
     */
    private function extractSldAndTldFromString($domain) {
        $getDomain = explode(".", $domain);
        $getSLD = $getDomain[0];
        $getTLD1 = $getDomain[1];
        $getTLD1 = "$getTLD1";
        $getTLD2 = "";
        if (isset($getDomain[2])) {
            $getTLD2 = ".".$getDomain[2];
        }
        $getTLD = "$getTLD1$getTLD2";

        $aReturn = array(
            'SLD' => $getSLD,
            'TLD' => $getTLD
        );

        return $aReturn;
    }
}
